/* Glipper - Clipboardmanager for Gnome
 * Copyright (C) 2006 Sven Rech <svenrech@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

//todo replace eggcellerator/keybinding with:
// http://developer.gnome.org/gtk2/2.24/gtk2-Keyboard-Accelerators.html
// see http://developer.gnome.org/gdk/stable/gdk-Windows.html#GdkModifierType 
 
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
				 
#include <gtk/gtk.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) gettext(String)
#ifdef gettext_noop
#define N_(String) gettext_noop(String)
#else
#define N_(String) (String)
#endif
#else /* NLS is disabled */
#define _(String) (String)
#define N_(String) (String)
#define textdomain(String) (String)
#define gettext(String) (String)
#define dgettext(Domain,String) (String)
#define dcgettext(Domain,String,Type) (String)
#define bindtextdomain(Domain,Directory) (Domain) 
#define bind_textdomain_codeset(Domain,Codeset) (Codeset) 
#endif /* ENABLE_NLS */

#define CHECK_INTERVAL 500 //The interval between the clipboard checks (ms)

//Preferences variables
int maxElements = 20; //Amount of elements in history
int maxItemLength = 35; //Length of one history entry
gboolean usePrimary = TRUE; //use Primary Clipboard
gboolean useDefault = TRUE;; //use Default Clipboard
gboolean markDefault = TRUE; //whether default entry should be tagged
gboolean weSaveHistory = TRUE; //whether history should be saved
char *keyComb;

GtkClipboard* PrimaryCl;
GtkClipboard* DefaultCl;
gchar* lastPr = NULL;
gchar* lastDf = NULL;
GSList* history = NULL;
GtkWidget* historyMenu = NULL;
GtkWidget* popupMenu;
GtkStatusIcon* TrayIcon;
GtkTooltips* toolTip; //The trayicon's tooltip
GtkWidget* eventbox; //The trayicon's eventbox
gint mainTimeout;
int hasChanged = 1;

void errorDialog(gchar* error_msg, gchar* secondaryText);
void saveHistory();

GtkWidget* historyLength;
GtkWidget* itemLength;
GtkWidget* primaryCheck;
GtkWidget* defaultCheck;
GtkWidget* markDefaultCheck;
GtkWidget* saveHistCheck;
GtkWidget* keyCombEntry;
GtkWidget* applyButton;
GtkWidget *preferences_dialog;


//Set markDefaultCheck checkbox (ctrl+c in blue) active or not
void
on_clipCheck_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	//Only permit to distinguish ctrl+c entry if both types of clipboard are present
	if (!(gtk_toggle_button_get_active((GtkToggleButton*)primaryCheck) && 
	  gtk_toggle_button_get_active((GtkToggleButton*)defaultCheck)))
		gtk_widget_set_sensitive(markDefaultCheck, FALSE);
	else
		gtk_widget_set_sensitive(markDefaultCheck, TRUE);
}


//Sets initial state of widgets
void setWidgets()
{
	gtk_spin_button_set_value((GtkSpinButton *) historyLength, maxElements);
	gtk_spin_button_set_value((GtkSpinButton *) itemLength, maxItemLength);
	gtk_toggle_button_set_active((GtkToggleButton*)primaryCheck, usePrimary);
	gtk_toggle_button_set_active((GtkToggleButton*)defaultCheck, useDefault);
	gtk_toggle_button_set_active((GtkToggleButton*)markDefaultCheck, markDefault);
	gtk_toggle_button_set_active((GtkToggleButton*)saveHistCheck, weSaveHistory);
	gtk_entry_set_text((GtkEntry*)keyCombEntry, keyComb);
	on_clipCheck_toggled(NULL, NULL); //can make markDefaultCheck possibly unsensitive
}

void historyEntryActivate(GtkMenuItem* menuItem, gpointer user_data)
{
	if (usePrimary)
		gtk_clipboard_set_text(PrimaryCl, (gchar*)user_data, -1);
	if (useDefault)
		gtk_clipboard_set_text(DefaultCl, (gchar*)user_data, -1);
	checkClipboard(NULL);
	hasChanged = 1;
}

//add history entry to menu:
GtkWidget* addHistMenuItem(gchar* item)
{
	static GtkTooltips* toolTip; //The tooltip for bold item

	if (toolTip == NULL) toolTip = gtk_tooltips_new();

	//we have to cut the string to "maxItemLength" characters:
	GString* ellipseData = g_string_new(item);
	if (ellipseData->len > maxItemLength)
	{
		ellipseData = g_string_erase(ellipseData, maxItemLength/2, 
									ellipseData->len-maxItemLength);
		ellipseData = g_string_insert(ellipseData, ellipseData->len/2, "...");
	}
	//remove special escape character (\n and \t):
	{ 
		int x;
		for (x=0; x<=ellipseData->len; x++)
		{
			if ((ellipseData->str[x]=='\n')||(ellipseData->str[x]=='\t'))
				ellipseData->str[x] = ' ';
		}
	}
	//The cutted string is now stored in "ellipseData"
	GtkWidget* MenuItem = gtk_menu_item_new_with_label(ellipseData->str);
	//if this string belongs to the default Clipboard, we tag it (presumed markDefault is true):
	if (markDefault && useDefault && usePrimary)
		if ((lastDf!=NULL)&&(g_ascii_strcasecmp(lastDf, item)==0))
		{
			GtkLabel* label = (GtkLabel*)gtk_bin_get_child((GtkBin*)MenuItem); 
			gchar* temp = g_strdup_printf("<b>%s</b>", ellipseData->str);
			gtk_label_set_markup(label, temp);
			g_free(temp);

			//Add description tooltip to bold entry
			gtk_tooltips_set_tip(toolTip, MenuItem, _("This entry was copied with ctrl+c.\nIt can be pasted with ctrl+v."), "Glipper");
		}
	g_string_free(ellipseData, TRUE);
	gtk_menu_append(historyMenu, MenuItem);
	g_signal_connect(G_OBJECT(MenuItem), "activate", 
							 G_CALLBACK(historyEntryActivate), 
							 item);
	return MenuItem;
}

void deleteHistory(GtkMenuItem* menuItem, gpointer user_data)
{
	hasChanged = 1;
	g_slist_free(history);
	history = NULL;
	if (weSaveHistory)
		saveHistory();
}


void createHistMenu()
{
	static GtkTooltips* toolTip = NULL; //The tooltip for first item
	static GtkWidget* menuHeader = NULL; //A nice header for the menu 

	if (toolTip == NULL) 
		toolTip = gtk_tooltips_new();

	//Create menu header
	//TODO : replace the menu item with non interactive widgets
	if (menuHeader == NULL)
		menuHeader = gtk_image_menu_item_new_from_stock(GTK_STOCK_PASTE,NULL);
		
	gtk_label_set_text((GtkLabel*)gtk_bin_get_child((GtkBin*)menuHeader), 
			g_strdup_printf(_("Glipper - Clipboardmanager  (%s)"), keyComb));

	if (historyMenu != NULL)
	{
		//We remove menuHeader from the old Menu. Before doing so,
		//we have to add a reference, for that it will not be deleted:
		gtk_widget_ref(menuHeader);  
		gtk_container_remove((GtkContainer*)historyMenu, menuHeader); 
		gtk_widget_destroy(historyMenu);
	}	
	historyMenu = gtk_menu_new();

	//Append menu header
	gtk_menu_append(historyMenu, menuHeader);
	gtk_menu_append(historyMenu, gtk_separator_menu_item_new());

	//Now add all history entrys:
	if (history == NULL)
		gtk_menu_append(historyMenu, 
			gtk_menu_item_new_with_label(_("<No Content>")));
	else
	{
		GSList* temp = history;

		//We add the first item manually
		GtkWidget* firstItem = addHistMenuItem(temp->data);
		//addHistMenuItem(temp->data);

		//Add description tooltip to first item
		gtk_tooltips_set_tip(toolTip, firstItem, _("This is the last element to be copied.\nIt can be pasted with the middle mouse button."), "Glipper");

		if (temp->next != NULL)
		{
			gtk_menu_append((GtkMenu*)historyMenu, gtk_separator_menu_item_new());
			while ((temp = g_slist_next(temp)) != NULL)
				addHistMenuItem(temp->data);
		}
	}

	//Add the "clear" button at the bottom of the menu
	gtk_menu_append((GtkMenu*)historyMenu, gtk_separator_menu_item_new());

	GtkWidget* deleteAll =
		gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR, NULL);
	g_signal_connect(G_OBJECT(deleteAll), "activate", G_CALLBACK(deleteHistory), NULL);
	gtk_menu_append((GtkMenu*)historyMenu, deleteAll);

	gtk_widget_show_all(historyMenu);
}

void keyhandler(char *keystring, gpointer user_data)
{
	if (hasChanged)
		createHistMenu();
	gtk_menu_popup ((GtkMenu*)historyMenu, NULL, NULL, NULL, NULL,
					0, gtk_get_current_event_time());
}


/*Makes sure, that the current preferences are used:*/
void applyPreferences()
{
	hasChanged=1; 
	GSList* deleteElement = g_slist_nth(history, maxElements-1);
	if (deleteElement != NULL)
	{
		g_slist_free(deleteElement->next);
		deleteElement->next = NULL;
	}
	keybinder_bind(keyComb, keyhandler, NULL);
	gtk_status_icon_set_tooltip_text(TrayIcon, g_strdup_printf(_("Glipper (%s)\nClipboardmanager"),
		keyComb));
}

//trys to open (or create) a file in "~/.glipper" for writing purposes:
FILE* writeGlipperFile(char* filename)
{
	gchar* directory = g_build_path("/", g_get_home_dir(), ".glipper", NULL);
	gchar* path = g_build_filename(directory, filename, NULL);
	FILE* file = fopen(path, "w");
	if (file == NULL)
		if (mkdir(directory, S_IRWXU)==0)  //Trys to create the folder if open failes
		{
			file = fopen(path, "w");
			if (file == NULL)
				g_warning("Can't open or create file %s!", file);
		}
		else
			g_warning ("Can't create directory '.glipper' in user's home directory!");
	g_free(directory);
	g_free(path);
	return file;
}

void savePreferences()
{
	FILE* prefFile = writeGlipperFile("prefs");
	if (prefFile != 0)
	{
		fwrite(&maxElements, sizeof(maxElements), 1, prefFile);
		fwrite(&maxItemLength, sizeof(maxItemLength), 1, prefFile);
		fwrite(&usePrimary, sizeof(usePrimary), 1, prefFile);
		fwrite(&useDefault, sizeof(useDefault), 1, prefFile);
		fwrite(&markDefault, sizeof(markDefault), 1, prefFile);
		fwrite(&weSaveHistory, sizeof(weSaveHistory), 1, prefFile);
		//this causes a segfault
		size_t len = strlen(keyComb);
		fwrite(&len, sizeof(size_t), 1, prefFile);
		fputs(keyComb, prefFile);
		fclose(prefFile);
	}
}



void unbindKey()
{
	keybinder_unbind(keyComb, keyhandler);
	free(keyComb);
	keyComb = NULL;
}


//Save preferences when window is closed via apply button
void
on_applyButton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	unbindKey();
	maxElements = gtk_spin_button_get_value_as_int((GtkSpinButton*)historyLength); 
	maxItemLength = gtk_spin_button_get_value_as_int((GtkSpinButton*)itemLength); 
	usePrimary = gtk_toggle_button_get_active((GtkToggleButton*)primaryCheck);
	useDefault = gtk_toggle_button_get_active((GtkToggleButton*)defaultCheck);
	markDefault = gtk_toggle_button_get_active((GtkToggleButton*)markDefaultCheck);
	weSaveHistory = gtk_toggle_button_get_active((GtkToggleButton*)saveHistCheck);
	keyComb = g_strdup(gtk_entry_get_text((GtkEntry*)keyCombEntry));
	savePreferences();
	applyPreferences();
	gtk_widget_destroy(preferences_dialog);
}



void showPreferences(gpointer data)
{
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *label2;
  GtkWidget *hbox15;
  GtkWidget *table1;
  GtkWidget *label8;
  GtkWidget *label9;
  GtkObject *historyLength_adj;
  GtkObject *itemLength_adj;
  GtkWidget *label3;
  GtkWidget *hbox18;
  GtkWidget *label10;
  GtkWidget *hbox16;
  GtkWidget *vbox8;
  GtkWidget *label5;
  GtkWidget *hbox17;
  GtkWidget *vbox9;
  GtkWidget *hbox19;
  GtkWidget *label11;
  GtkWidget *dialog_action_area1;
  GtkWidget *applyButton;

  preferences_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (preferences_dialog), 5);
  gtk_window_set_title (GTK_WINDOW (preferences_dialog), _("Glipper Preferences"));
  gtk_window_set_resizable (GTK_WINDOW (preferences_dialog), FALSE);
  gtk_window_set_icon_name (GTK_WINDOW (preferences_dialog), "gtk-preferences");
  gtk_window_set_type_hint (GTK_WINDOW (preferences_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_dialog_set_has_separator (GTK_DIALOG (preferences_dialog), FALSE);

  vbox1 = GTK_DIALOG (preferences_dialog)->vbox;
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 5);


  label2 = gtk_label_new (_("<b>Limits</b>"));

  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label2), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  hbox15 = gtk_hbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox1), hbox15, TRUE, TRUE, 0);

  table1 = gtk_table_new (2, 2, FALSE);

  gtk_box_pack_start (GTK_BOX (hbox15), table1, FALSE, FALSE, 18);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 6);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 12);

  label8 = gtk_label_new (_("Amount of entries in history:"));

  gtk_table_attach (GTK_TABLE (table1), label8, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label9 = gtk_label_new (_("Length of one history entry:"));

  gtk_table_attach (GTK_TABLE (table1), label9, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  historyLength_adj = gtk_adjustment_new (1, 1, 99, 1, 10, 0);
  historyLength = gtk_spin_button_new (GTK_ADJUSTMENT (historyLength_adj), 1, 0);

  gtk_table_attach (GTK_TABLE (table1), historyLength, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  itemLength_adj = gtk_adjustment_new (1, 10, 99, 1, 10, 0);
  itemLength = gtk_spin_button_new (GTK_ADJUSTMENT (itemLength_adj), 1, 0);

  gtk_table_attach (GTK_TABLE (table1), itemLength, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label3 = gtk_label_new (_("<b>Clipboards</b>"));

  gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label3), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  hbox18 = gtk_hbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox1), hbox18, FALSE, FALSE, 0);

  label10 = gtk_label_new (_("Which clipboards should be managed by Glipper?"));
  gtk_box_pack_start (GTK_BOX (hbox18), label10, FALSE, FALSE, 18);
  gtk_label_set_use_markup (GTK_LABEL (label10), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);

  hbox16 = gtk_hbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox1), hbox16, TRUE, TRUE, 0);

  vbox8 = gtk_vbox_new (FALSE, 6);

  gtk_box_pack_start (GTK_BOX (hbox16), vbox8, TRUE, TRUE, 18);

  primaryCheck = gtk_check_button_new_with_mnemonic (_("Select - Mark/middle mouse button"));

  gtk_box_pack_start (GTK_BOX (vbox8), primaryCheck, FALSE, FALSE, 0);

  defaultCheck = gtk_check_button_new_with_mnemonic (_("Copy - Ctrl+C/Ctrl+V"));

  gtk_box_pack_start (GTK_BOX (vbox8), defaultCheck, FALSE, FALSE, 0);


  label5 = gtk_label_new (_("<b>Behaviour</b>"));

  gtk_box_pack_start (GTK_BOX (vbox1), label5, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label5), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  hbox17 = gtk_hbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox1), hbox17, TRUE, TRUE, 0);

  vbox9 = gtk_vbox_new (FALSE, 6);

  gtk_box_pack_start (GTK_BOX (hbox17), vbox9, TRUE, TRUE, 18);

  markDefaultCheck = gtk_check_button_new_with_mnemonic (_("Tag Ctrl+C clipboard's content in history (bold)"));

  gtk_box_pack_start (GTK_BOX (vbox9), markDefaultCheck, FALSE, FALSE, 0);

  saveHistCheck = gtk_check_button_new_with_mnemonic (_("Save history"));

  gtk_box_pack_start (GTK_BOX (vbox9), saveHistCheck, FALSE, FALSE, 0);

  hbox19 = gtk_hbox_new (FALSE, 3);

  gtk_box_pack_start (GTK_BOX (vbox9), hbox19, FALSE, FALSE, 0);

  label11 = gtk_label_new (_("Key combination for popup: "));
  gtk_widget_show (label11);
  gtk_box_pack_start (GTK_BOX (hbox19), label11, FALSE, FALSE, 0);

  keyCombEntry = gtk_entry_new ();

  gtk_box_pack_start (GTK_BOX (hbox19), keyCombEntry, TRUE, TRUE, 0);
  gtk_entry_set_text (GTK_ENTRY (keyCombEntry), g_strdup(keyComb));
  gtk_entry_set_invisible_char (GTK_ENTRY (keyCombEntry), 9679);

  dialog_action_area1 = GTK_DIALOG (preferences_dialog)->action_area;

  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  applyButton = gtk_button_new_from_stock ("gtk-apply");

  gtk_dialog_add_action_widget (GTK_DIALOG (preferences_dialog), applyButton, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (applyButton, GTK_CAN_DEFAULT);

	//Connect signals to handlers
	g_signal_connect_after ((gpointer) primaryCheck, "toggled",
		G_CALLBACK (on_clipCheck_toggled),
                          NULL);
	g_signal_connect_after ((gpointer) defaultCheck, "toggled",
		G_CALLBACK (on_clipCheck_toggled),
		NULL);
	g_signal_connect ((gpointer) applyButton, "clicked",
		G_CALLBACK (on_applyButton_clicked),
		NULL);

	//Show preferences dialog
	setWidgets();
	gtk_widget_show_all(preferences_dialog);

	//free the glade data
}


void getClipboards()
{
	/*There exists two different Clipboards:
	  PrimaryClipboard: Marked with mouse
	  DefaultClipboard: copied with Strg+C
	*/
	DefaultCl = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
	PrimaryCl = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
}





void insertInHistory(gchar* content)
{
	if ((history == NULL) ||
	  (g_ascii_strcasecmp(content, (gchar*)history->data)!=0))
	{
		//g_print(content);
		history = g_slist_prepend(history, g_strdup(content));
		hasChanged = 1;
		//We look whether the same entry still exists in the history and delete it:
		GSList* temp = history;
		while ((temp->next!=NULL)&&(temp->next->data!=NULL)&&
		  (g_ascii_strcasecmp((gchar*)temp->next->data, content)!=0))
		  	temp = temp->next;
		//If the same entry was found:
		if (temp->next!=NULL)
		{
			GSList* dummy = temp->next;
			temp->next = temp->next->next;
			g_slist_free_1(dummy);
		}
		//We shorten the history if it gets longer than "maxElements":
		GSList* deleteElement = g_slist_nth(history, maxElements-1);
		if (deleteElement != NULL)
		{
			g_slist_free(deleteElement->next);
			deleteElement->next = NULL;
		}
		if (weSaveHistory)
			saveHistory();
	}
}

void processContent(gchar* newContent, gchar** lastContent, GtkClipboard* Clipboard)
{
	if (newContent == NULL)
	{
		if (*lastContent != NULL)
			gtk_clipboard_set_text(Clipboard, *lastContent, -1);
	} else {
		if ((*lastContent==NULL)||
		  ((*lastContent!=NULL)&&(g_ascii_strcasecmp(newContent, *lastContent)!=0)))
		{
			insertInHistory(newContent);
			if (*lastContent!=NULL)
				g_free(*lastContent);
			*lastContent = newContent;
		}
		else
			g_free(newContent);
	}
}

gboolean checkClipboard(gpointer data)
{
	g_source_remove(mainTimeout);
	if (usePrimary)
	{
		gchar* newContentPr = gtk_clipboard_wait_for_text(PrimaryCl);
		processContent(newContentPr, &lastPr, PrimaryCl);
	}
	if (useDefault)
	{
		gchar* newContentCl = gtk_clipboard_wait_for_text(DefaultCl);
		processContent(newContentCl, &lastDf, DefaultCl);
	}
	mainTimeout = g_timeout_add(500, checkClipboard, NULL);
	return 1;
}

void TrayIconClicked(GtkWidget* widget, GdkEventButton *event, gpointer user_data)
{
	if (event->type != GDK_BUTTON_PRESS)
		return;
	switch (event->button)
	{
		case 1:
			if (hasChanged)
				createHistMenu();
			hasChanged = 0;
			gtk_menu_popup ((GtkMenu*)historyMenu, NULL, NULL, NULL, NULL,
				1, gtk_get_current_event_time());
			break;
		case 3:
			gtk_menu_popup((GtkMenu*)popupMenu, NULL, NULL, NULL, NULL, 
				1, gtk_get_current_event_time());
			break;
	}
}


void createTrayIcon()
{
	//Load trayicon
	TrayIcon = gtk_status_icon_new_from_stock(GTK_STOCK_PASTE);
	gtk_status_icon_set_tooltip_text(GTK_STATUS_ICON(TrayIcon),
		g_strdup_printf(_("Glipper (%s)\nClipboardmanager"),
		keyComb));
	g_signal_connect_swapped(G_OBJECT(TrayIcon), "button-press-event", 
							 G_CALLBACK(TrayIconClicked), NULL);
}

void show_about(gpointer data)
{
	gchar* authors[] = {"Sven Rech <svenrech@gmx.de>", "Karderio <karderio at gmail dot com>", NULL};

	gchar* license = 
"This library is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU Lesser General Public\n"
"License as published by the Free Software Foundation; either\n"
"version 2.1 of the License, or (at your option) any later version.\n"
"\n"
"This library is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
"Lesser General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU Lesser General Public\n"
"License along with this library; if not, write to the Free Software\n"
"Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA";

	GError* pix_error = NULL;
	gtk_show_about_dialog(NULL, 
		"authors", authors,
		"copyright", "Copyright © 2006 Sven Rech",
		"translator-credits", _("translator-credits"),
		"license", license,
		"name", "Glipper",
		"comments", _("Clipboardmanager for Gnome"),
		"logo-icon-name", 	"gtk-paste",
		"website", "http://glipper.sourceforge.net/",
		"version", VERSION,
		 NULL);
}

//Creates context menu that we popup on right click on the trayicon
void createPopupMenu()
{
	//Create new popup menu
	popupMenu = gtk_menu_new();

	//Create widgets to be plced in the popup menu
	GtkWidget* quit = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	g_signal_connect(G_OBJECT(quit), "activate", G_CALLBACK(gtk_main_quit), NULL);

	GtkWidget* about = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
	g_signal_connect(G_OBJECT(about), "activate", G_CALLBACK(show_about), NULL);

	GtkWidget* preferences = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
	g_signal_connect(G_OBJECT(preferences), "activate", G_CALLBACK(showPreferences), NULL);

	//Add the widgets to the menu
	gtk_menu_append((GtkMenu*)popupMenu, preferences);
	gtk_menu_append((GtkMenu*)popupMenu, about);
	gtk_menu_append((GtkMenu*)popupMenu, gtk_separator_menu_item_new());
	gtk_menu_append((GtkMenu*)popupMenu, quit);

	gtk_widget_show_all(popupMenu);
}




void saveElem(gpointer data, gpointer histFile)
{
	GString* entry = g_string_new((gchar*)data);
	fwrite(&(entry->len), 4, 1, (FILE*)histFile);
	fputs(entry->str, (FILE*)histFile);
	g_string_free(entry, TRUE);
}

void saveHistory()
{
	FILE* histFile = writeGlipperFile("history");
	if (histFile != NULL)
	{
		fputc(g_slist_length(history), histFile);
		g_slist_foreach(history, saveElem, histFile);
		fclose(histFile);
	}
}

void readHistory()
{
	gchar* path= g_build_filename(g_get_home_dir(), ".glipper/history", NULL);
	FILE* histFile = fopen(path, "r");
	g_free(path);
	if (histFile != 0)
	{	
		int length = fgetc(histFile);
		int x;
		for (x=0; x < length; x++)
		{
			int size;
			fread(&size, 4, 1, histFile);
			gchar* data = (gchar*)g_malloc(size+1);
			fread(data, size, 1, histFile);
			data[size] = '\0';
			history = g_slist_append(history, data);
		}
	}
}

void readPreferences()
{
	gchar* path= g_build_filename(g_get_home_dir(), ".glipper/prefs", NULL);
	FILE* prefFile = fopen(path, "r");
	g_free(path);
	free(keyComb);
	if (prefFile != 0)
	{
		fread(&maxElements, sizeof(maxElements), 1, prefFile);
		fread(&maxItemLength, sizeof(maxItemLength), 1, prefFile);
		fread(&usePrimary, sizeof(usePrimary), 1, prefFile);
		fread(&useDefault, sizeof(useDefault), 1, prefFile);
		fread(&markDefault, sizeof(markDefault), 1, prefFile);
		fread(&weSaveHistory, sizeof(weSaveHistory), 1, prefFile);
		size_t len;
		fread(&len, sizeof(size_t), 1, prefFile);
		if (!feof(prefFile))
		{
			keyComb = malloc(len+1);
			fgets(keyComb, 80, prefFile);
			fclose(prefFile);
			return;
		}
		fclose(prefFile);
	}
	keyComb = malloc(strlen("<Ctrl><Alt>c")+1);
	strcpy(keyComb, "<Ctrl><Alt>c");
}

int main(int argc, char *argv[])
{
	//gettext configuration
	setlocale("C", "");
	bindtextdomain (GETTEXT_PACKAGE, GLIPPERLOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	gtk_init (&argc, &argv);

	getClipboards();
	readPreferences();

	//Setup keyboard shortcut
	keybinder_init();
	keybinder_bind(keyComb, keyhandler, NULL);

	createTrayIcon();
	createPopupMenu();

	if (weSaveHistory)
		readHistory();

	mainTimeout = g_timeout_add(CHECK_INTERVAL, checkClipboard, NULL);
	gtk_main ();

	unbindKey();

	return 0;
}
